#include "InputContext.h"
#include "Variant.h"
#include <iostream>

using namespace std;

class PyZyObserver : public PyZy::InputContext::Observer {
public:
    void commitText (PyZy::InputContext *context, const string &commit_text) {
        m_commited_text = commit_text;
    }

    void inputTextChanged (PyZy::InputContext *context) {}
    void preeditTextChanged (PyZy::InputContext *context) {}
    void auxiliaryTextChanged (PyZy::InputContext *context) {}
    void cursorChanged (PyZy::InputContext *context) {}
    void candidatesChanged (PyZy::InputContext *context) {}

    const string& commitedText () { return m_commited_text; }

    void clear () {
        m_commited_text.clear ();
    }

private:
    string m_commited_text;
};

PyZyObserver observer;
PyZy::Candidate candidate;

extern "C"
{
    void* initContext(const string & user_cache_dir,
                    const string & user_config_dir) {
	PyZy::InputContext::init(user_cache_dir,user_config_dir);
        PyZy::InputContext *context = PyZy::InputContext::create(PyZy::InputContext::BOPOMOFO, &observer);
	context->setProperty (PyZy::InputContext::PROPERTY_SPECIAL_PHRASE,PyZy::Variant::fromBool (false));
        context->setProperty (PyZy::InputContext::PROPERTY_BOPOMOFO_SCHEMA,PyZy::Variant::fromUnsignedInt (0));
        context->setProperty (PyZy::InputContext::PROPERTY_MODE_SIMP,PyZy::Variant::fromBool (false));
	return context;
    }

    bool insertChar(void* obj, char ch) {
        PyZy::InputContext *context = reinterpret_cast<PyZy::InputContext*>(obj);
        return context->insert(ch);
    }

    bool removeChar(void* obj) {
        PyZy::InputContext *context = reinterpret_cast<PyZy::InputContext*>(obj);
        return context->removeCharBefore();
    }

    bool hasCandidate(void* obj, int i) {
        PyZy::InputContext *context = reinterpret_cast<PyZy::InputContext*>(obj);
        return context->hasCandidate(i);
    }

    bool selectCandidate(void* obj, int i) {
        PyZy::InputContext *context = reinterpret_cast<PyZy::InputContext*>(obj);
        observer.clear();
        return context->selectCandidate(i);
    }

    int getCandidatesSize(void* obj) {
        PyZy::InputContext *context = reinterpret_cast<PyZy::InputContext*>(obj);
        return context->getPreparedCandidatesSize();
    }

    const string& getCandidate(void* obj, int i) {
        PyZy::InputContext *context = reinterpret_cast<PyZy::InputContext*>(obj);
        context->getCandidate(i, candidate);
        return candidate.text;
    }

    const string& restText (void* obj) {
        PyZy::InputContext *context = reinterpret_cast<PyZy::InputContext*>(obj);
	return context->restText();
    }

    const string& auxiliaryText (void* obj) {
        PyZy::InputContext *context = reinterpret_cast<PyZy::InputContext*>(obj);
	return context->auxiliaryText();
    }

    const string& conversionText (void* obj) {
        PyZy::InputContext *context = reinterpret_cast<PyZy::InputContext*>(obj);
        return context->conversionText();
    }

    const string& selectedText(void* obj) {
        PyZy::InputContext *context = reinterpret_cast<PyZy::InputContext*>(obj);
        return context->selectedText();
    }

    const string& commitText (void* obj) {
        PyZy::InputContext *context = reinterpret_cast<PyZy::InputContext*>(obj);
	return observer.commitedText();
    }

    void resetContext(void* obj) {
        PyZy::InputContext *context = reinterpret_cast<PyZy::InputContext*>(obj);
        context->reset();
    }

    void close (void* obj) {
	PyZy::InputContext *context = reinterpret_cast<PyZy::InputContext*>(obj);
        PyZy::InputContext::finalize();
	delete context;
    }
}
